@@ -5,4 +5,4 @@
  
 """
 
-from . import core,id_converter
\ No newline at end of file
+from . import core,id_converter,integration
\ No newline at end of file
